//
//  CloverConnect.swift
//  clover_connect
//
//  Created by Phan.dev
//

import Flutter
import UIKit
import CloverConnector

class CloverConnect: DefaultCloverConnectorListener, PairingDeviceConfiguration {
    var fCloverConnector: ICloverConnector? // cloverConnector: ICloverConnector variable is existed in this class
    var currentStatus: String?
    var authToken: String?
    var pairingCode: String?
    
    func connect(connectionConfig: [String: String]) {
        // load from previous pairing, or nil will force/require
        // a new pairing with the device
        let savedAuthToken = loadAuthToken()
        let config = WebSocketDeviceConfiguration(
            endpoint: connectionConfig["endpoint"] as! String,
            remoteApplicationID: connectionConfig["remoteApplicationID"] as! String,
            posName: connectionConfig["posName"] as! String,
            posSerial: connectionConfig["posSerial"] as! String,
            pairingAuthToken: savedAuthToken,
            pairingDeviceConfiguration: self
        )
        
        fCloverConnector = CloverConnectorFactory.createICloverConnector(config: config)
        fCloverConnector?.addCloverConnectorListener(self)
        fCloverConnector?.initializeConnection()
    }
    
    func disconnect() {
        fCloverConnector?.showMessage(Messages.Clover.disconnecting)
        self.authToken = nil
        fCloverConnector?.resetDevice()
        fCloverConnector?.dispose()
    }

    // store the token to be loaded later by loadAuthToken
    func saveAuthToken(token: String) {
        authToken = token
    }
    
    func loadAuthToken() -> String? { return authToken }
    
    // PairingDeviceConfiguration
    func onPairingCode(_ pairingCode: String) {
        // display pairingCode to user, to be entered on the Clover Mini
        self.currentStatus = Events.onPairingCode
        self.pairingCode = pairingCode
    }
    
    func onPairingSuccess(_ authToken: String) {
        // pairing is successful
        // save this authToken to pass in to the config for future connections
        // so pairing will happen automatically
        self.currentStatus = Events.onPairingSuccess
        saveAuthToken(token: authToken)
    }
    
    func doSale(saleRequest: [String: Any]) {
        fCloverConnector?.showMessage(Messages.Clover.saleProcessing)
        // if onDeviceReady has been called
        let amount = saleRequest["amount"] as! Int
        let externalId = saleRequest["externalId"] as! String
        let sr = SaleRequest(amount: amount, externalId: externalId)
        
        // configure other properties of SaleRequest
        fCloverConnector?.sale(sr)
    }
    
    // DefaultCloverConnectorListener
    
    // called when device is disconnected
    override func onDeviceDisconnected() {
        self.currentStatus = Events.onDeviceDisconnected
    }
    
    // called when device is connected, but not ready for requests
    override func onDeviceConnected() {
        self.currentStatus = Events.onDeviceConnected
        fCloverConnector?.showWelcomeScreen()
    }
    
    // called when device is ready to take requests. Note: May be called more than once
    override func onDeviceReady(_ info:MerchantInfo){
        self.currentStatus = Events.onDeviceReady
    }
    
    // required if Mini wants the POS to verify a signature
    override func onVerifySignatureRequest(_ signatureVerifyRequest: VerifySignatureRequest) {
        self.currentStatus = Events.onVerifySignatureRequest
        //present signature to user, then
        // acceptSignature(...) or rejectSignature(...)
        fCloverConnector?.acceptSignature(signatureVerifyRequest)
    }
    
    // required if Mini wants the POS to verify a payment
    override func onConfirmPaymentRequest(_ request: ConfirmPaymentRequest) {
        self.currentStatus = Events.onConfirmPaymentRequest
        //present 1 or more challenges to user, then
        fCloverConnector?.acceptPayment(request.payment!)
        // or
        // fCloverConnector?.rejectPayment(...)
    }
    
    // override other callback methods
    override func onSaleResponse(_ response:SaleResponse) {
        self.currentStatus = Events.onSaleResponse
        if response.success {
            // sale successful and payment is in the response (response.payment)
        } else {
            // sale failed or was canceled
        }
    }
    
    override func onAuthResponse(_ response:AuthResponse) {
        self.currentStatus = Events.onAuthResponse
    }
    override func onPreAuthResponse(_ response:PreAuthResponse) {
        self.currentStatus = Events.onPreAuthResponse
    }
    
    // will provide UI information about the activity on the Mini,
    // and may provide input options for the POS to select some
    // options on behalf of the customer
    // see CloverConnectorListener.swift for example of calling invokeInputOption from this callback
    // etc.
    override func onDeviceActivityStart(_ deviceEvent:CloverDeviceEvent){
        self.currentStatus = Events.onDeviceActivityStart
    }
    override func onDeviceActivityEnd(_ deviceEvent:CloverDeviceEvent){
        self.currentStatus = Events.onDeviceActivityEnd
    }
}
